package ch.rakudave.jnetmap.plugins;

import javax.swing.Icon;

import ch.rakudave.jnetmap.view.preferences.PreferencePanel;

/**
 * Denotes a jNetMap-Plugin. Must offer information that will be displayed in the preferences and the respective menus.
 * Note: All Plugins MUST also annotate their base-class with @PluginImplementation, otherwise it will not be loaded
 * @author rakudave
 */
public interface Plugin extends net.xeoh.plugins.base.Plugin {
	/**
	 * @return the name of this plugins
	 */
	public String getPluginName();
	
	/**
	 * @return the author of this plugin
	 */
	public String getAuthor();
	
	/**
	 * @return the description of this plugin, displayed in the settings window
	 */
	public String getDescription();
	
	/**
	 * @return the icon of this plugin 
	 * (hint: you may use the built-in Base64-decoder so you don't have to ship extra resources)
	 */
	public Icon getIcon();
	
	/**
	 * @return true if this plugins offers a settings-panel
	 * @see getSettingPanel
	 */
	public boolean hasSettings();
	
	/**
	 * @return the settings-panel of this plugins that should be displayed in the settings window, may return null if hasSettings() is false 
	 */
	public PreferencePanel getSettingsPanel();
}
